#import "igXMLParser.h"


@implementation igXMLParser
{

}

@synthesize xmlParser, rootElement, currentElementPointer;

- (void)parserDidStartDocument:(NSXMLParser *)parser
{
    self.rootElement = nil;
    self.currentElementPointer = nil;
}

- (void)parser:(NSXMLParser *)parser didStartElement:(NSString *)elementName namespaceURI:(NSString *)namespaceURI qualifiedName:(NSString *)qName attributes:(NSDictionary *)attributeDict
{
    if (self.rootElement == nil)
    {
        self.rootElement = [[igXMLElement alloc] init];
        self.currentElementPointer = self.rootElement;
    }
    else
    {
        igXMLElement *newElement = [[igXMLElement alloc] init];
        newElement.parent = self.currentElementPointer;
        [self.currentElementPointer.subElements addObject:newElement];
        self.currentElementPointer = newElement;
    }

    self.currentElementPointer.name = elementName;
    self.currentElementPointer.attributes = attributeDict;
}

- (void)parser:(NSXMLParser *)parser foundCharacters:(NSString *)string
{
    if ([self.currentElementPointer.text length] > 0)
    {
        self.currentElementPointer.text = [self.currentElementPointer.text stringByAppendingString:string];
    }
    else
    {
        self.currentElementPointer.text = string;
    }
}

- (void)parser:(NSXMLParser *)parser didEndElement:(NSString *)elementName namespaceURI:(NSString *)namespaceURI qualifiedName:(NSString *)qName
{
    self.currentElementPointer = self.currentElementPointer.parent;
}

- (void)parserDidEndDocument:(NSXMLParser *)parser
{
    self.currentElementPointer = nil;
}

- (igXMLElement *)parseXmlWithData:(NSData *)data
{

    self.xmlParser = [[NSXMLParser alloc] initWithData:data];
    self.xmlParser.delegate = self;
    if ([self.xmlParser parse])
    {
        //XML Parsed
        return rootElement;
    }
    else
    {
        //XML Parse Failed
        return nil;
    }
}

- (igXMLElement *)parseXmlWithUrl:(NSURL *)urlString
{
    self.xmlParser = [[NSXMLParser alloc] initWithContentsOfURL:urlString];
    self.xmlParser.delegate = self;
    if ([self.xmlParser parse])
    {
        //XML Parsed
        return rootElement;
    }
    else
    {
        //XML Parse Failed
        return nil;
    }
}


@end

@implementation igXMLElement
{

}

@synthesize name, text, attributes, subElements, parent;

- (NSMutableArray *)subElements
{
    if (subElements == nil)
    {
        subElements = [[NSMutableArray alloc] init];
    }
    return subElements;
}

- (NSMutableArray *)findElementByName:(NSString *)elementName
{
    NSMutableArray *results = [[NSMutableArray alloc] init];
    [self searchElement:self.subElements searchResults:results searchName:elementName];
    return results;
}

- (void)searchElement:(NSMutableArray *)elements searchResults:(NSMutableArray *)results searchName:(NSString *)searchName
{
    for (igXMLElement *xmlElement in elements)
    {
        if ([xmlElement.name isEqualToString:searchName])
        {
            [results addObject:xmlElement];
        }

        if ([xmlElement.subElements count] > 0)
        {
            [self searchElement:xmlElement.subElements searchResults:results searchName:searchName];
        }
    }

}


@end